// Definitions

/// {@template state_transition_entry}
/// Entry for [StateGraph], contains optional [transition] and [sideEffect].
/// {@endtemplate}
class StateTransitionEntry<InState, Event, OutState> {
  /// State to new state conversion.
  final StateTransition<InState, Event, OutState> transition;

  /// Side effect of transition (e.g. logging).
  final SideEffect<InState, Event> sideEffect;

  /// {@macro state_transition_entry}
  StateTransitionEntry({this.transition, this.sideEffect});
}

/// Transition builder function.
typedef OutState StateTransition<InState, Event, OutState>(
    InState state, Event event);

/// Side effect builder function.
typedef void SideEffect<InState, Event>(InState state, Event event);

// Sugar functions

/// Makes [StateTransitionEntry] with just state-to-state [transition].
StateTransitionEntry<InState, Event, OutState>
    transition<InState, Event, OutState>(
  StateTransition<InState, Event, OutState> transition,
) {
  return StateTransitionEntry<InState, Event, OutState>(transition: transition);
}

/// Makes [StateTransitionEntry] with both [transition] and [sideEffect].
StateTransitionEntry<InState, Event, OutState>
    transitionWithSideEffect<InState, Event, OutState>(
  StateTransition<InState, Event, OutState> transition,
  SideEffect<InState, Event> sideEffect,
) {
  return StateTransitionEntry<InState, Event, OutState>(
    transition: transition,
    sideEffect: sideEffect,
  );
}

/// Makes [StateTransitionEntry] with just [sideEffect] of transition.
StateTransitionEntry<InState, Event, InState> sideEffect<InState, Event>(
  SideEffect<InState, Event> action,
) {
  return StateTransitionEntry<InState, Event, InState>(sideEffect: action);
}
